@@ -13,13 +13,15 @@ import logging
 from typing import Any, Dict, Optional, Text
 from typing import List  # noqa pylint: disable=unused-import
 
-from alexapy.aiohttp import ClientConnectionError, ClientResponse
 import backoff
 from yarl import URL
 
+from alexapy.aiohttp import ClientConnectionError, ClientResponse
+
 from .alexalogin import AlexaLogin
 from .errors import (
     AlexapyConnectionError,
+    AlexapyLoginCloseRequested,
     AlexapyLoginError,
     AlexapyTooManyRequestsError,
 )
@@ -85,6 +87,17 @@ class AlexaAPI:
         #               uri,
         #               data,
         #               query)
+        if self._login.close_requested:
+            _LOGGER.debug(
+                "Login object has been asked to close; ignoring %s request to %s with %s %s",
+                method,
+                uri,
+                data,
+                query,
+            )
+            raise AlexapyLoginCloseRequested()
+        if self._session.closed:
+            raise AlexapyLoginError("Session is closed")
         response = await getattr(self._session, method)(
             url,
             json=data,
@@ -107,7 +120,7 @@ class AlexaAPI:
         return response
 
     async def _post_request(
-        self, uri: Text, data: Optional[Dict[Text, Text]] = None
+        self, uri: Text, data: Optional[Dict[Text, Any]] = None
     ) -> ClientResponse:
         return await self._request("post", uri, data)
 
@@ -149,6 +162,17 @@ class AlexaAPI:
         #               uri,
         #               data,
         #               query)
+        if login.close_requested:
+            _LOGGER.debug(
+                "Login object has been asked to close; ignoring %s request to %s with %s %s",
+                method,
+                uri,
+                data,
+                query,
+            )
+            raise AlexapyLoginCloseRequested()
+        if session.closed:
+            raise AlexapyLoginError("Session is closed")
         response = await getattr(session, method)(
             url,
             json=data,
@@ -267,6 +291,8 @@ class AlexaAPI:
         **kwargs : Each named variable must match a recognized Amazon variable
                    within the operationPayload. Please see examples in
                    play_music, send_announcement, and send_tts.
+                   Variables with value None are removed from the operationPayload.
+                   Variables prefixed with "root_" will be added to the root node instead.
 
         Supported sequences:
         Alexa.Weather.Play
@@ -292,8 +318,15 @@ class AlexaAPI:
             "locale": (self._device._locale if self._device._locale else "en-US"),
             "customerId": self._device._device_owner_customer_id,
         }
+        root_node = {}
         if kwargs is not None:
             operation_payload.update(kwargs)
+            for key, value in kwargs.items():
+                if value is None:  # remove null keys
+                    operation_payload.pop(key)
+                elif isinstance(value, str) and value.startswith("root_"):
+                    operation_payload.pop(key)
+                    root_node[key] = value[5:]
             if kwargs.get("devices"):
                 operation_payload.pop("deviceType")
                 operation_payload.pop("deviceSerialNumber")
@@ -302,6 +335,7 @@ class AlexaAPI:
             "type": sequence,
             "operationPayload": operation_payload,
         }
+        node_data.update(root_node)
         await self.run_behavior(node_data, queue_delay=queue_delay)
 
     @_catch_all_exceptions
@@ -484,7 +518,7 @@ class AlexaAPI:
             skillId="amzn1.ask.1p.alexadevicecontrols",
             customerId=customer_id,
             queue_delay=queue_delay,
-            **kwargs
+            **kwargs,
         )
 
     def process_targets(
@@ -657,7 +691,7 @@ class AlexaAPI:
         customer_id: Text = None,
         queue_delay: float = 1.5,
     ) -> None:
-        """Send announcment to Alexa devices.
+        """Send mobile push to Alexa app.
 
         Push a message to mobile devices with the Alexa App. This probably
         should be a static method.
@@ -687,6 +721,47 @@ class AlexaAPI:
             queue_delay=queue_delay,
         )
 
+    @_catch_all_exceptions
+    async def send_dropin_notification(
+        self,
+        message: Text,
+        title: Text = "AlexaAPI Dropin Notification",
+        customer_id: Text = None,
+        queue_delay: float = 1.5,
+    ) -> None:
+        """Send dropin notification to Alexa app for Alexa device.
+
+        Push a message to mobile devices with the Alexa App. This can spawn a
+        notification to drop in on a specific device.
+
+        Args:
+        message (string): The message to push to the mobile device.
+        title (string): Title for push notification
+        customer_id (string): CustomerId to use for sending. When none
+                              specified this defaults to the device owner.
+        queue_delay (float, optional): The number of seconds to wait
+                                        for commands to queue together.
+                                        Defaults to 1.5.
+                                        Must be positive.
+
+        """
+        await self.send_sequence(
+            "Alexa.Notifications.DropIn",
+            customerId=(
+                customer_id
+                if customer_id is not None
+                else self._device._device_owner_customer_id
+            ),
+            notificationMessage=message,
+            alexaUrl="#v2/comms/conversation-list?showDropInDialog=true",
+            title=title,
+            skillId="root_amzn1.ask.1p.action.dropin",
+            queue_delay=queue_delay,
+            deviceType=None,
+            deviceSerialNumber=None,
+            locale=None,
+        )
+
     async def set_media(self, data: Dict[Text, Any]) -> None:
         """Select the media player."""
         await self._post_request(
@@ -751,7 +826,7 @@ class AlexaAPI:
         await self.set_media({"type": "RepeatCommand", "repeat": setting})
 
     @_catch_all_exceptions
-    async def get_state(self) -> Dict[Text, Any]:
+    async def get_state(self) -> Optional[Dict[Text, Any]]:
         """Get playing state."""
         response = await self._get_request(
             "/api/np/player?deviceSerialNumber="
@@ -786,7 +861,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_bluetooth(login) -> Dict[Text, Any]:
+    async def get_bluetooth(login) -> Optional[Dict[Text, Any]]:
         """Get paired bluetooth devices."""
         response = await AlexaAPI._static_request(
             "get", login, "/api/bluetooth", query={"cached": "false"}
@@ -817,7 +892,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_devices(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_devices(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Identify all Alexa devices."""
         response = await AlexaAPI._static_request(
             "get", login, "/api/devices-v2/device", query=None
@@ -831,7 +906,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_authentication(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_authentication(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Get authentication json."""
         response = await AlexaAPI._static_request(
             "get", login, "/api/bootstrap", query=None
@@ -844,7 +919,9 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_activities(login: AlexaLogin, items: int = 10) -> Dict[Text, Any]:
+    async def get_activities(
+        login: AlexaLogin, items: int = 10
+    ) -> Optional[Dict[Text, Any]]:
         """Get activities json."""
         response = await AlexaAPI._static_request(
             "get",
@@ -858,7 +935,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_device_preferences(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_device_preferences(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Identify all Alexa device preferences."""
         response = await AlexaAPI._static_request(
             "get", login, "/api/device-preferences", query={}
@@ -867,7 +944,9 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_automations(login: AlexaLogin, items: int = 1000) -> Dict[Text, Any]:
+    async def get_automations(
+        login: AlexaLogin, items: int = 1000
+    ) -> Optional[Dict[Text, Any]]:
         """Identify all Alexa automations."""
         response = await AlexaAPI._static_request(
             "get", login, "/api/behaviors/automations", query={"limit": items}
@@ -929,7 +1008,9 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_guard_state(login: AlexaLogin, entity_id: Text) -> Dict[Text, Any]:
+    async def get_guard_state(
+        login: AlexaLogin, entity_id: Text
+    ) -> Optional[Dict[Text, Any]]:
         """Get state of Alexa guard.
 
         Args:
@@ -951,7 +1032,7 @@ class AlexaAPI:
     @_catch_all_exceptions
     async def static_set_guard_state(
         login: AlexaLogin, entity_id: Text, state: Text
-    ) -> Dict[Text, Any]:
+    ) -> Optional[Dict[Text, Any]]:
         """Set state of Alexa guard.
 
         Args:
@@ -984,7 +1065,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_guard_details(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_guard_details(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Get Alexa Guard details.
 
         Args:
@@ -1004,7 +1085,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_notifications(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_notifications(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Get Alexa notifications.
 
         Args:
@@ -1024,7 +1105,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def set_notifications(login: AlexaLogin, data) -> Dict[Text, Any]:
+    async def set_notifications(login: AlexaLogin, data) -> Optional[Dict[Text, Any]]:
         """Update Alexa notification.
 
         Args:
@@ -1043,7 +1124,7 @@ class AlexaAPI:
 
     @staticmethod
     @_catch_all_exceptions
-    async def get_dnd_state(login: AlexaLogin) -> Dict[Text, Any]:
+    async def get_dnd_state(login: AlexaLogin) -> Optional[Dict[Text, Any]]:
         """Get Alexa DND states.
 
         Args:
@@ -1102,3 +1183,32 @@ class AlexaAPI:
                     "%s:Succesfully deleted %s", hide_email(email), activity["id"],
                 )
         return completed
+
+    @_catch_all_exceptions
+    async def set_background(self, url: Text) -> bool:
+        """Set background for Echo Show.
+
+        Sets the background to Alexa App Photo with the specific https url.
+
+        Args
+        url (URL): valid https url for the image
+
+        Returns
+        Whether the command was successful.
+
+        """
+        data = {
+            "deviceSerialNumber": self._device.unique_id,
+            "deviceType": self._device._device_type,
+            "backgroundImageID": "JqIFZhtBTx25wLGTJGdNGQ",
+            "backgroundImageType": "PERSONAL_PHOTOS",
+            "backgroundImageURL": url,
+        }
+        _LOGGER.debug("Setting background of %s to: %s", self._device, url)
+        if url.startswith("http://"):
+            _LOGGER.warning("Background URL should be a valid https image")
+        response = await self._post_request("/api/background-image", data=data)
+        response_json = await response.json(content_type=None) if response else None
+        success = response.status == 200
+        _LOGGER.debug("Success: %s Response: %s", success, response_json)
+        return success
