@@ -8,19 +8,19 @@ For more details about this api, please refer to the documentation at
 https://gitlab.com/keatontaylor/alexapy
 """
 
+from json import JSONDecodeError
 import logging
-from typing import (
-    Callable,
-    Dict,  # noqa pylint: disable=unused-import
-    List,
-    Optional,
-    Text,
-    Tuple,
-    Union,
-)
+import re
+from typing import Callable, List, Optional, Text, Tuple, Union
+from typing import Dict  # noqa pylint: disable=unused-import
+from urllib.parse import urlencode
+
+from bs4 import BeautifulSoup
+from simplejson import JSONDecodeError as SimpleJSONDecodeError
+from urllib.parse import urlparse
 
 from alexapy import aiohttp
-from bs4 import BeautifulSoup
+from alexapy.aiohttp.client_exceptions import ContentTypeError
 
 from .const import EXCEPTION_TEMPLATE
 from .helpers import _catch_all_exceptions
@@ -79,10 +79,11 @@ class AlexaLogin:
         self._options: Optional[Dict[Text, Text]] = {}
         self._site: Optional[Text] = None
         self._create_session()
+        self._close_requested = False
 
     @property
     def email(self) -> Text:
-        """Return email for this Login."""
+        """Return email or mobile account for this Login."""
         return self._email
 
     @property
@@ -92,9 +93,14 @@ class AlexaLogin:
 
     @property
     def url(self) -> Text:
-        """Return session for this Login."""
+        """Return url for this Login."""
         return self._url
 
+    @property
+    def close_requested(self) -> bool:
+        """Return whether this Login has been asked to close."""
+        return self._close_requested
+
     @property
     def links(self) -> Text:
         """Return string list of links from last page for this Login."""
@@ -199,6 +205,7 @@ class AlexaLogin:
 
     async def close(self) -> None:
         """Close connection for login."""
+        self._close_requested = True
         if self._session and not self._session.closed:
             if self._session._connector_owner:
                 assert self._session._connector is not None
@@ -218,6 +225,7 @@ class AlexaLogin:
         self._options = {}
         self._site = None
         self._create_session()
+        self._close_requested = False
         import os
         from aiofiles import os as aioos
 
@@ -273,10 +281,6 @@ class AlexaLogin:
             self._prefix + self._url + "/api/bootstrap", cookies=cookies, ssl=self._ssl
         )
         await self._process_resp(get_resp)
-        from simplejson import JSONDecodeError as SimpleJSONDecodeError
-        from json import JSONDecodeError
-        from aiohttp.client_exceptions import ContentTypeError
-
         try:
             json = await get_resp.json()
             email = json["authentication"]["customerEmail"]
@@ -286,9 +290,12 @@ class AlexaLogin:
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             return False
-        if email.lower() == self._email.lower():
+        if email != "" and email.lower() == self._email.lower():
             _LOGGER.debug("Logged in as %s", email)
             return True
+        if email == "":
+            _LOGGER.debug("Logged in as mobile account %s", email)
+            return True
         _LOGGER.debug("Not logged in due to email mismatch")
         await self.reset()
         return False
@@ -298,9 +305,9 @@ class AlexaLogin:
             #  define session headers
             self._headers = {
                 "User-Agent": (
-                    "Mozilla/5.0 (Windows NT 6.3; Win64; x64) "
+                    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) "
                     "AppleWebKit/537.36 (KHTML, like Gecko) "
-                    "Chrome/68.0.3440.106 Safari/537.36"
+                    "Chrome/83.0.4103.116 Safari/537.36"
                 ),
                 "Accept": (
                     "text/html,application/xhtml+xml, "
@@ -364,13 +371,15 @@ class AlexaLogin:
         # pylint: disable=too-many-statements
         """Login to Amazon."""
         data = data or {}
-        if cookies and await self.test_loggedin(cookies):
+        if cookies:
             _LOGGER.debug("Using cookies to log in")
-            self.status = {}
-            self.status["login_successful"] = True
-            _LOGGER.debug("Log in successful with cookies")
-            self._prepare_cookies_from_session(self._url)
-            return
+            if await self.test_loggedin(cookies):
+                self.status = {}
+                self.status["login_successful"] = True
+                _LOGGER.debug("Log in successful with cookies")
+                self._prepare_cookies_from_session(self._url)
+                return
+            await self.reset()
         _LOGGER.debug("No valid cookies for log in; using credentials")
         #  site = 'https://www.' + self._url + '/gp/sign-in.html'
         #  use alexa site instead
@@ -417,7 +426,10 @@ class AlexaLogin:
         # This commented block can be used to read a file directly to process.
         # import aiofiles
 
-        # async with aiofiles.open("/config/anti-automation.html", "rb") as myfile:
+        # async with aiofiles.open("/config/anti-automation-js.html", "rb") as myfile:
+        # async with aiofiles.open(
+        #     "/config/Amazon-Password-Assistance.html", "rb"
+        # ) as myfile:
         #     html = await myfile.read()
         site = await self._process_page(html, site)
         if not self.status.get("ap_error"):
@@ -427,16 +439,23 @@ class AlexaLogin:
 
                 if missing_params:
                     _LOGGER.debug(
-                        "WARNING: Detected missing params: %s", missing_params
+                        "WARNING: Detected missing params: %s",
+                        [k for (k, v) in self._data.items() if v == ""],
                     )
                 _LOGGER.debug("Session Cookies:\n%s", self._print_session_cookies())
                 _LOGGER.debug("Submit Form Data: %s", dumps(self._data))
                 _LOGGER.debug("Header: %s", dumps(self._headers))
 
             # submit post request with username/password and other needed info
-            post_resp = await self._session.post(
-                site, data=self._data, headers=self._headers, ssl=self._ssl,
-            )
+            if self.status.get("force_get"):
+                post_resp = await self._session.get(
+                    site, data=self._data, headers=self._headers, ssl=self._ssl,
+                )
+            else:
+                post_resp = await self._session.post(
+                    site, data=self._data, headers=self._headers, ssl=self._ssl,
+                )
+
             # headers need to be submitted to have the referer
             if self._debug:
                 import aiofiles
@@ -487,8 +506,8 @@ class AlexaLogin:
 
         def find_links() -> None:
             links = {}
+            index = 0
             if links_tag:
-                index = 0
                 for link in links_tag:
                     if not link.string:
                         continue
@@ -503,6 +522,28 @@ class AlexaLogin:
                     elif href.startswith("http"):
                         links[str(index)] = (string, href)
                         index += 1
+            if forms_tag:
+                for form in forms_tag:
+                    if (
+                        form.get("method")
+                        and form.get("method") == "get"
+                        and form.get("action")
+                    ):
+                        string = form.get("id")
+                        action = form.get("action")
+                        params = {}
+                        inputs = form.findAll("input")
+                        for item in inputs:
+                            if (
+                                item
+                                and item.get("type")
+                                and item.get("type") == "hidden"
+                            ):
+                                params[item.get("name")] = item.get("value")
+                        href = f"{self._prefix}{self._url}{action}?{urlencode(params)}"
+                        links[str(index)] = (string, href)
+                        index += 1
+            if links:
                 _LOGGER.debug("Links: %s", links)
             self._links = links
 
@@ -524,9 +565,13 @@ class AlexaLogin:
         authselect_tag = soup.find("form", {"id": "auth-select-device-form"})
         verificationcode_tag = soup.find("form", {"action": "verify"})
         verification_captcha_tag = soup.find("img", {"alt": "captcha"})
+        javascript_authentication_tag = soup.find("form", {"id": "pollingForm"})
         links_tag = soup.findAll("a", href=True)
+        forms_tag = soup.findAll("form")
         form_tag = soup.find("form")
         missingcookies_tag = soup.find(id="ap_error_return_home")
+        forgotpassword_tag = soup.find("form", {"name": "forgotPassword"})
+        polling_tag = soup.find(id="updatedChannelDetails")
         if self._debug:
             find_links()
 
@@ -630,7 +675,35 @@ class AlexaLogin:
             for link in links:
                 href = link["href"]
             status["ap_error"] = True
+            status["force_get"] = True
             status["ap_error_href"] = href
+        elif javascript_authentication_tag:
+            message: Text = ""
+
+            message = soup.find("span").getText()
+            for div in soup.findAll("div", {"id": "channelDetails"}):
+                message += div.getText()
+            status["force_get"] = True
+            status["message"] = re.sub("(\\s)+", "\\1", message)
+            _LOGGER.debug("Javascript Authentication page detected: %s", message)
+        elif forgotpassword_tag:
+            status["message"] = (
+                "Forgot password page detected; "
+                "Amazon has detected too many failed logins. "
+                "Please check to see if Amazon requires any further action. "
+                "You may have to wait before retrying."
+            )
+            status["ap_error"] = True
+            _LOGGER.warning(status["message"])
+            status["login_failed"] = "forgot_password"
+        elif polling_tag:
+            status["force_get"] = True
+            status["message"] = self.status["message"]
+            approval_status = soup.find("input", {"id": "transactionApprovalStatus"})
+            _LOGGER.debug(
+                "Polling page detected: %s with %s", polling_tag, approval_status
+            )
+            status["approval_status"] = approval_status.get("value")
         else:
             _LOGGER.debug("Captcha/2FA not requested; confirming login.")
             if await self.test_loggedin():
@@ -664,7 +737,7 @@ class AlexaLogin:
 
             else:
                 _LOGGER.debug("Login failed; check credentials")
-                status["login_failed"] = True
+                status["login_failed"] = "login_failed"
                 assert self._data is not None
                 if "" in self._data.values():
                     missing = [k for (k, v) in self._data.items() if v == ""]
@@ -675,13 +748,13 @@ class AlexaLogin:
                     )
         self.status = status
         # determine post url if not logged in
-        if form_tag and "login_successful" not in status:
+        if status.get("approval_status") == "TransactionCompleted":
+            site = self._data.get("openid.return_to")
+        elif form_tag and "login_successful" not in status:
             formsite: Text = form_tag.get("action")
             if self._debug:
                 _LOGGER.debug("Found form to process: %s", form_tag)
             if formsite and formsite == "verify":
-                import re
-
                 search_results = re.search(r"(.+)/(.*)", str(site))
                 assert search_results is not None
                 site = search_results.groups()[0] + "/verify"
@@ -694,7 +767,13 @@ class AlexaLogin:
                     site = self._headers["Referer"]
                 _LOGGER.debug("Found post url to get; forcing get to %s", site)
                 self._lastreq = None
-            elif formsite and formsite != "get":
+            elif formsite and formsite == "/ap/cvf/approval/poll":
+                self._data = self.get_inputs(soup, {"id": "pollingForm"})
+                url = urlparse(site)
+                site = f"{url.scheme}://{url.netloc}{formsite}"
+                # site = form_tag.find("input", {"name": "openid.return_to"}).get("value")
+                _LOGGER.debug("Found url for polling page %s", site)
+            elif formsite:
                 site = formsite
                 _LOGGER.debug("Found post url to %s", site)
         return site
@@ -718,7 +797,7 @@ class AlexaLogin:
         verificationcode: Optional[Text] = (
             None if "verificationcode" not in data else data["verificationcode"]
         )
-        _LOGGER.debug("Preparing post to %s with input data: %s", site, data)
+        _LOGGER.debug("Preparing form submission to %s with input data: %s", site, data)
 
         #  add username and password to the data for post request
         #  check if there is an input field
