@@ -9,14 +9,19 @@ Helpers.
 For more details about this api, please refer to the documentation at
 https://gitlab.com/keatontaylor/alexapy
 """
-import logging
 from asyncio import CancelledError
+from http.cookies import CookieError
 from json import JSONDecodeError
+import logging
 
-from alexapy.aiohttp import ClientConnectionError
+from alexapy.aiohttp import ClientConnectionError, ContentTypeError
 
 from .const import EXCEPTION_TEMPLATE
-from .errors import AlexapyConnectionError, AlexapyLoginError
+from .errors import (
+    AlexapyConnectionError,
+    AlexapyLoginCloseRequested,
+    AlexapyLoginError,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -24,14 +29,16 @@ _LOGGER = logging.getLogger(__name__)
 def hide_email(email):
     """Obfuscate email."""
     part = email.split("@")
-    return "{}{}{}@{}{}{}".format(
-        part[0][0],
-        "*" * (len(part[0]) - 2),
-        part[0][-1],
-        part[1][0],
-        "*" * (len(part[1]) - 2),
-        part[1][-1],
-    )
+    if len(part) > 1:
+        return "{}{}{}@{}{}{}".format(
+            part[0][0],
+            "*" * (len(part[0]) - 2),
+            part[0][-1],
+            part[1][0],
+            "*" * (len(part[1]) - 2),
+            part[1][-1],
+        )
+    return hide_serial(email)
 
 
 def hide_serial(item):
@@ -45,6 +52,7 @@ def hide_serial(item):
                 "deviceSerialNumber",
                 "serialNumber",
                 "destinationUserId",
+                "customerId",
             ]:
                 response[key] = hide_serial(value)
     elif isinstance(item, str):
@@ -59,6 +67,40 @@ def hide_serial(item):
     return response
 
 
+def obfuscate(item):
+    """Obfuscate email, password, and other known sensitive keys."""
+    if item is None:
+        return ""
+    if isinstance(item, dict):
+        response = item.copy()
+        for key, value in item.items():
+            if key in ["password"]:
+                response[key] = f"REDACTED {len(value)} CHARS"
+            elif key in ["email"]:
+                response[key] = hide_email(value)
+            elif key in [
+                "deviceSerialNumber",
+                "serialNumber",
+                "destinationUserId",
+                "customerId",
+            ]:
+                response[key] = hide_serial(value)
+            elif isinstance(value, (dict, list, tuple)):
+                response[key] = obfuscate(value)
+    elif isinstance(item, (list, tuple)):
+        response = []
+        for list_item in item:
+            if isinstance(list_item, (dict, list, tuple)):
+                response.append(obfuscate(list_item))
+            else:
+                response.append(list_item)
+        if isinstance(item, tuple):
+            response = tuple(response)
+    else:
+        return item
+    return response
+
+
 def _catch_all_exceptions(func):
     # pylint: disable=import-outside-toplevel
     import functools
@@ -72,38 +114,50 @@ def _catch_all_exceptions(func):
                 "%s.%s(%s, %s): A connection error occured: %s",
                 func.__module__[func.__module__.find(".") + 1 :],
                 func.__name__,
-                args,
-                kwargs,
+                obfuscate(args),
+                obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             raise AlexapyConnectionError
-        except (JSONDecodeError) as ex:
+        except (JSONDecodeError, CookieError) as ex:
             _LOGGER.error(
                 "%s.%s(%s, %s): A login error occured: %s",
                 func.__module__[func.__module__.find(".") + 1 :],
                 func.__name__,
-                args,
-                kwargs,
+                obfuscate(args),
+                obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             raise AlexapyLoginError
-        except CancelledError as ex:  # pylint: disable=broad-except
+        except (ContentTypeError) as ex:
+            _LOGGER.error(
+                "%s.%s(%s, %s): A login error occured; Amazon may want you to change your password: %s",
+                func.__module__[func.__module__.find(".") + 1 :],
+                func.__name__,
+                obfuscate(args),
+                obfuscate(kwargs),
+                EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
+            )
+            raise AlexapyLoginError
+        except CancelledError as ex:
             _LOGGER.warning(
                 "%s.%s(%s, %s): Timeout error occured accessing AlexaAPI: %s",
                 func.__module__[func.__module__.find(".") + 1 :],
                 func.__name__,
-                args,
-                kwargs,
+                obfuscate(args),
+                obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             return None
+        except AlexapyLoginCloseRequested:
+            raise
         except Exception as ex:  # pylint: disable=broad-except
             _LOGGER.error(
                 "%s.%s(%s, %s): An error occured accessing AlexaAPI: %s",
                 func.__module__[func.__module__.find(".") + 1 :],
                 func.__name__,
-                args,
-                kwargs,
+                obfuscate(args),
+                obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
             raise
